@@ -16,8 +16,6 @@
 .. module:: interp
 """
 
-from __future__ import print_function
-
 import numpy as np
 import scipy.linalg
 import galsim
@@ -182,7 +180,7 @@ class BasisInterp(Interp):
         A = np.vstack(A_chunks)
         b = np.concatenate(b_chunks)
 
-        if A.shape[0] < A.shape[1]:
+        if len(stars) < len(K) or A.shape[0] < A.shape[1]:
             raise RuntimeError("Too few constraints for solution. (Probably too few stars)")
 
         # Note: The following snippet is the straightforward way to do this using the
@@ -374,10 +372,9 @@ class BasisPolynomial(BasisInterp):
             # Exception if we have any requests for negative orders
             raise ValueError('Negative polynomial order specified')
 
-        # TODO: Need to update the Interp write command to handle lists.
-        #       Or write a custom BasisPolynomial.write function.
         self.kwargs = {
             'order' : order,
+            'keys' : keys,
             'use_qr' : use_qr
         }
 
@@ -392,6 +389,12 @@ class BasisPolynomial(BasisInterp):
     def getProperties(self, star):
         return np.array([star.data[k] for k in self._keys], dtype=float)
 
+    @property
+    def property_names(self):
+        """List of properties used by this interpolant.
+        """
+        return self._keys
+
     def basis(self, star):
         """Return 1d array of polynomial basis values for this star
 
