@@ -16,8 +16,6 @@
 .. module:: input
 """
 
-from __future__ import print_function
-from past.builtins import basestring
 import numpy as np
 import scipy
 import glob
@@ -93,36 +91,16 @@ class Input(object):
         else:
             logger.debug("Making star list from %d catalogs", self.nimages)
 
-        args = [(self.__class__,
-                 self.image_kwargs[k], self.cat_kwargs[k], self.wcs_list[k], self.chipnums[k])
+        args = [(self.image_kwargs[k], self.cat_kwargs[k], self.wcs_list[k], self.chipnums[k])
                 for k in range(self.nimages)]
-        kwargs = dict(stamp_size=self.stamp_size, min_snr=self.min_snr, max_snr=self.max_snr,
+        kwargs = dict(stamp_size=self.stamp_size,
                       pointing=self.pointing, use_partial=self.use_partial,
                       invert_weight=self.invert_weight,
-                      remove_signal_from_weight=self.remove_signal_from_weight,
-                      hsm_size_reject=self.hsm_size_reject,
-                      max_mask_pixels=self.max_mask_pixels,
-                      max_edge_frac=self.max_edge_frac,
-                      stamp_center_size=self.stamp_center_size)
+                      remove_signal_from_weight=self.remove_signal_from_weight)
 
-        all_stars = run_multi(call_makeStarsFromImage, self.nproc, raise_except=True,
+        all_stars = run_multi(self._makeStarsFromImage, self.nproc, raise_except=True,
                               args=args, logger=logger, kwargs=kwargs)
 
-        # Apply the reserve separately on each ccd, so they each reserve 20% of their stars
-        # (or whatever fraction).  We wouldn't want to accidentally reserve all the stars on
-        # one of the ccds by accident, for instance.
-        if self.reserve_frac != 0:
-            for stars in all_stars:
-                if stars is None or len(stars) == 0:
-                    continue
-                # Mark a fraction of the stars as reserve stars
-                nreserve = int(self.reserve_frac * len(stars))  # round down
-                logger.info("Reserve %s of %s (reserve_frac=%s) input stars",
-                            nreserve, len(stars), self.reserve_frac)
-                reserve_list = self.rng.choice(len(stars), nreserve, replace=False)
-                for i, star in enumerate(stars):
-                    star.data.properties['is_reserve'] = i in reserve_list
-
         # Concatenate the star lists into a single list
         stars = [s for slist in all_stars if slist is not None for s in slist if slist]
 
@@ -151,7 +129,7 @@ class Input(object):
 
 
 class InputFiles(Input):
-    """An Input handler than just takes a list of image files and catalog files.
+    """An Input handler that just takes a list of image files and catalog files.
     """
     def __init__(self, config, logger=None):
         """
@@ -201,7 +179,7 @@ class InputFiles(Input):
                 `https://github.com/GalSim-developers/GalSim/wiki/Config-Values`_
 
         There are many other optional parameters, which help govern how the input files are
-        read or interporeted:
+        read or interpreted:
 
             :chipnum:       The id number of this chip used to reference this image [default:
                             image_num]
@@ -230,6 +208,8 @@ class InputFiles(Input):
                             Items with flag & skip_flag != 0 will be skipped.
             :use_flag:      The flag indicating which items to use. [default: None]
                             Items with flag & use_flag == 0 will be skipped.
+            :property_cols: A list of column names of star properties (e.g. star colors).
+                            [default: None]
             :sky_col:       The name of a column with sky values. [default: None]
             :gain_col:      The name of a column with gain values. [default: None]
             :sky:           The sky level to subtract from the image values. [default: None]
@@ -241,40 +221,12 @@ class InputFiles(Input):
                             for the Poisson noise from the galaxy flux.
             :satur:         The staturation level.  If any pixels for a star exceed this, then
                             the star is skipped. [default: None]
-            :min_snr:       The minimum S/N ratio to use.  If an input star is too faint, it is
-                            removed from the input list of PSF stars.
-            :max_snr:       The maximum S/N ratio to allow for any given star.  If an input star
-                            is too bright, it can have too large an influence on the interpolation,
-                            so this parameter limits the effective S/N of any single star.
-                            Basically, it adds noise to bright stars to lower their S/N down to
-                            this value.  [default: 100]
-            :max_edge_frac: Cutoff on the fraction of the flux comming from pixels on the edges of
-                            the postage stamp. [default: None]
-            :stamp_center_size: Distance from center of postage stamp (in pixels) to consider as
-                            defining the edge of the stamp for the purpose of the max_edge_fact cut.
-                            The default value of 13 is most of the radius of a 32x32 stamp size.
-                            If you change stamp_size, you should consider what makes sense here.
-                            [default 13].
-            :max_mask_pixels: If given, reject stars with more than this many masked pixels
-                            (i.e. those with w=0). [default: None]
             :use_partial:   Whether to use stars whose postage stamps are only partially on the
                             full image.  [default: False]
-            :hsm_size_reject: Whether to reject stars with a very different hsm-measured size than
-                            the other stars in the input catalog.  (Used to reject objects with
-                            neighbors or other junk in the postage stamp.) [default: False]
-                            If this is a float value, it gives the number of inter-quartile-ranges
-                            to use for rejection relative to the median.  hsm_size_reject=True
-                            is equivalent to hsm_size_reject=10.
             :nstars:        Stop reading the input file at this many stars.  (This is applied
                             separately to each input catalog.)  [default: None]
             :nproc:         How many multiprocessing processes to use for reading in data from
                             multiple files at once. [default: 1]
-            :reserve_frac:  Reserve a fraction of the stars from the PSF calculations, so they
-                            can serve as fair points for diagnostic testing.  These stars will
-                            not be used to constrain the PSF model, but the output files will
-                            contain the reserve stars, flagged as such.  Generally 0.2 is a
-                            good choice if you are going to use this. [default: 0.]
-            :seed:          A seed to use for numpy.random.default_rng, if desired. [default: None]
 
             :wcs:           Normally, the wcs is automatically read in when reading the image.
                             However, this parameter allows you to optionally provide a different
@@ -288,7 +240,7 @@ class InputFiles(Input):
                             stamps may be smaller than this if the star is near a chip boundary.
                             [default: 32]
             :ra, dec:       The RA, Dec of the telescope pointing. [default: None; See
-                            :setPointing: for details about how this can be specified]
+                            `setPointing` for details about how this can be specified]
 
 
         :param config:      The configuration dict used to define the above parameters.
@@ -309,6 +261,7 @@ class InputFiles(Input):
                 'dec_col' : str,
                 'ra_units' : str,
                 'dec_units' : str,
+                'property_cols' : list,
                 'sky_col' : str,
                 'gain_col' : str,
                 'flag_col' : str,
@@ -317,24 +270,18 @@ class InputFiles(Input):
                 'image_hdu' : int,
                 'weight_hdu' : int,
                 'badpix_hdu' : int,
+                'sky_file_name' : str,
+                'sky_hdu': int,
                 'cat_hdu' : int,
                 'invert_weight' : bool,
                 'remove_signal_from_weight' : bool,
                 'stamp_size' : int,
                 'gain' : str,
                 'satur' : str,
-                'min_snr' : float,
-                'max_snr' : float,
                 'use_partial' : bool,
-                'hsm_size_reject' : float,
-                'max_edge_frac': float,
-                'stamp_center_size': float,
-                'max_mask_pixels' : int,
                 'sky' : str,
                 'noise' : float,
                 'nstars' : int,
-                'reserve_frac' : float,
-                'seed' : int,
               }
         ignore = [ 'nproc', 'nimages', 'ra', 'dec', 'wcs' ]  # These are parsed separately
 
@@ -369,7 +316,7 @@ class InputFiles(Input):
             if nimages < 1:
                 raise ValueError('input.nimages must be >= 1')
 
-        # Deal with dir here, since sometimes we need to have it already atteched for glob
+        # Deal with dir here, since sometimes we need to have it already attached for glob
         # to work.
         if 'dir' in config:
             dir = galsim.config.ParseValue(config, 'dir', base, str)[0]
@@ -383,7 +330,7 @@ class InputFiles(Input):
                 raise ValueError("image_file_name may not be an empty list")
             if dir is not None:
                 image_list = [os.path.join(dir, n) for n in image_list]
-        elif isinstance(config['image_file_name'], basestring):
+        elif isinstance(config['image_file_name'], str):
             image_file_name = config['image_file_name']
             if dir is not None:
                 image_file_name = os.path.join(dir, image_file_name)
@@ -412,6 +359,23 @@ class InputFiles(Input):
         logger.debug('nimages = %d',nimages)
         assert nimages is not None
 
+        # Set up sky_file_name if appropriate
+        if 'sky_file_name' not in config:
+            pass
+        elif isinstance(config['sky_file_name'], str):
+            sky_file_name = config['sky_file_name']
+            if dir is not None:
+                sky_file_name = os.path.join(dir, sky_file_name)
+            sky_list = sorted(glob.glob(sky_file_name))
+            if len(sky_list) == 0:
+                raise ValueError("No files found corresponding to "+config['sky_file_name'])
+            config['sky_file_name'] = {
+                'type' : 'List_str',
+                'items' : sky_list
+            }
+        elif not isinstance(config['sky_file_name'], dict):
+            raise ValueError("sky_file_name should be either a dict or a string")
+
         if 'cat_file_name' not in config:
             raise TypeError('Parameter cat_file_name is required')
         elif isinstance(config['cat_file_name'], list):
@@ -420,14 +384,14 @@ class InputFiles(Input):
                 raise ValueError("cat_file_name may not be an empty list")
             if dir is not None:
                 cat_list = [os.path.join(dir, n) for n in cat_list]
-        elif isinstance(config['cat_file_name'], basestring):
+        elif isinstance(config['cat_file_name'], str):
             cat_file_name = config['cat_file_name']
             if dir is not None:
                 cat_file_name = os.path.join(dir, cat_file_name)
             cat_list = sorted(glob.glob(cat_file_name))
             if len(cat_list) == 0:
                 raise ValueError("No files found corresponding to "+config['cat_file_name'])
-        elif not isinstance(config['cat_file_name'], dict):
+        elif not isinstance(config['cat_file_name'], (dict, type(None))):
             raise ValueError("cat_file_name should be either a dict or a string")
 
         if cat_list is not None:
@@ -453,12 +417,6 @@ class InputFiles(Input):
 
         self.remove_signal_from_weight = config.get('remove_signal_from_weight', False)
         self.invert_weight = config.get('invert_weight', False)
-        self.reserve_frac = config.get('reserve_frac', 0.)
-        try:
-            self.rng = np.random.default_rng(config.get('seed', None))
-        except AttributeError:  # pragma: no cover
-            # numpy <= 1.16 doesn't have this yet.  But RandomState is fine.
-            self.rng = np.random.RandomState(config.get('seed', None))
 
         logger.info("Reading in %d images",nimages)
         for image_num in range(nimages):
@@ -479,6 +437,8 @@ class InputFiles(Input):
             image_hdu = params.get('image_hdu', None)
             weight_hdu = params.get('weight_hdu', None)
             badpix_hdu = params.get('badpix_hdu', None)
+            sky_file_name = params.get('sky_file_name', None)
+            sky_hdu = params.get('sky_hdu', None)
             noise = params.get('noise', None)
 
             self.image_file_name.append(image_file_name)
@@ -487,6 +447,8 @@ class InputFiles(Input):
                     'image_hdu' : image_hdu,
                     'weight_hdu' : weight_hdu,
                     'badpix_hdu' : badpix_hdu,
+                    'sky_file_name' : sky_file_name,
+                    'sky_hdu' : sky_hdu,
                     'noise' : noise})
 
             # Read the catalog
@@ -501,6 +463,7 @@ class InputFiles(Input):
             flag_col = params.get('flag_col', None)
             skip_flag = params.get('skip_flag', -1)
             use_flag = params.get('use_flag', None)
+            property_cols = params.get('property_cols', None)
             sky_col = params.get('sky_col', None)
             gain_col = params.get('gain_col', None)
             sky = params.get('sky', None)
@@ -526,6 +489,7 @@ class InputFiles(Input):
                     'flag_col' : flag_col,
                     'skip_flag' : skip_flag,
                     'use_flag' : use_flag,
+                    'property_cols': property_cols,
                     'sky_col' : sky_col,
                     'gain_col' : gain_col,
                     'sky' : sky,
@@ -535,18 +499,7 @@ class InputFiles(Input):
                     'image_file_name' : image_file_name,
                     'stamp_size' : self.stamp_size})
 
-        self.min_snr = config.get('min_snr', None)
-        self.max_snr = config.get('max_snr', 100)
-        self.max_edge_frac = config.get('max_edge_frac', None)
-        self.max_mask_pixels = config.get('max_mask_pixels', None)
-        self.stamp_center_size = config.get('stamp_center_size', 13)
-
         self.use_partial = config.get('use_partial', False)
-        self.hsm_size_reject = config.get('hsm_size_reject', 0.)
-        if self.hsm_size_reject == 1:
-            # Enable True to be equivalent to 10.  True comes in as 1.0, which would be a
-            # silly value to use, so it shouldn't be a problem to turn 1.0 -> 10.0.
-            self.hsm_size_reject = 10.
 
         # Read all the wcs's, since we'll need this for the pointing, which in turn we'll
         # need for when we make the stars.
@@ -557,6 +510,45 @@ class InputFiles(Input):
         dec = config.get('dec',None)
         self.setPointing(ra, dec, logger)
 
+    def load_images(self, stars, logger=None):
+        """Load the image data into a list of Stars.
+
+        We don't store the image data for Stars when we write them to a file, since that
+        would take up a lot of space and is usually not desired.  However, we do store the
+        bounds in the original image where the star was cutout, so if you want to load back in
+        the original data from the image file(s), you can do so with this function.
+
+        :param stars:           A list of Star instances.
+        :param logger:          A logger object for logging debug info. [default: None]
+
+        :returns: a new list of Stars with the images information loaded.
+        """
+        logger = galsim.config.LoggerWrapper(logger)
+
+        images = {}
+        weight_images = {}
+        for image_num in range(self.nimages):
+            image, weight, _, _ = self.getRawImageData(image_num)
+            images[self.chipnums[image_num]] = image
+            weight_images[self.chipnums[image_num]] = weight
+
+        loaded_stars = []
+        for star in stars:
+            chipnum = star['chipnum']
+            image = images[chipnum][star.data.image.bounds].copy()
+            if 'sky' in star.data.properties:
+                image -= star['sky']
+            weight = weight_images[chipnum][star.data.weight.bounds].copy()
+            data = StarData(image=image,
+                            image_pos=star.data.image_pos,
+                            weight=weight,
+                            pointing=star.data.pointing,
+                            properties=star.data.properties,
+                            _xyuv_set=True)
+            loaded_stars.append(Star(data=data, fit=star.fit))
+
+        return loaded_stars
+
     def getRawImageData(self, image_num, logger=None):
         return self._getRawImageData(self.image_kwargs[image_num], self.cat_kwargs[image_num],
                                      self.wcs_list[image_num], self.invert_weight,
@@ -575,21 +567,22 @@ class InputFiles(Input):
         # Update the wcs
         image.wcs = wcs
 
-        image_pos, sky, gain, satur = InputFiles.readStarCatalog(
+        image_pos, extra_props = InputFiles.readStarCatalog(
                 logger=logger, image=image, **cat_kwargs)
 
         if remove_signal_from_weight:
             # Subtract off the mean sky, since this isn't part of the "signal" we want to
             # remove from the weights.
-            if sky is None:
-                signal = image
+            if 'sky' in extra_props:
+                signal = image - np.mean(extra_props['sky'])
             else:
-                signal = image - np.mean(sky)
+                signal = image
             # For the gain, either all are None or all are values.
+            gain = extra_props['gain']
             if gain[0] is None:
                 # If None, then we want to estimate the gain from the weight image.
                 weight, g = InputFiles._removeSignalFromWeight(signal, weight)
-                gain = [g for _ in gain]
+                extra_props['gain'] = [g for _ in gain]
                 logger.warning("Empirically determined gain = %f",g)
             else:
                 # If given, use the mean gain when removing the signal.
@@ -598,29 +591,20 @@ class InputFiles(Input):
                 weight, _ = InputFiles._removeSignalFromWeight(signal, weight, gain=np.mean(gain))
             logger.info("Removed signal from weight image.")
 
-        return image, weight, image_pos, sky, gain, satur
+        return image, weight, image_pos, extra_props
 
     @staticmethod
     def _makeStarsFromImage(image_kwargs, cat_kwargs, wcs, chipnum,
-                            stamp_size, min_snr, max_snr, pointing, use_partial,
-                            invert_weight, remove_signal_from_weight, hsm_size_reject,
-                            max_mask_pixels, max_edge_frac, stamp_center_size,
+                            stamp_size, pointing, use_partial,
+                            invert_weight, remove_signal_from_weight,
                             logger):
-        """Make stars from a single input image
+        """Make "stars" from a single input image
         """
-        image, wt, image_pos, sky, gain, satur = InputFiles._getRawImageData(
+        image, wt, image_pos, extra_props = InputFiles._getRawImageData(
                 image_kwargs, cat_kwargs, wcs, invert_weight, remove_signal_from_weight, logger)
-        logger.info("Processing catalog %s with %d stars",chipnum,len(image_pos))
+        logger.info("Processing catalog %s with %d objects",chipnum,len(image_pos))
 
-        nstars_in_image = 0
-        stars = []
-
-        if max_edge_frac is not None:
-            cen = (stamp_size-1.)/2.            # index at center of array.  May be half-integral.
-            i,j = np.ogrid[0:stamp_size,0:stamp_size]
-            edge_mask = (i-cen)**2 + (j-cen)**2 > stamp_center_size**2
-        else:
-            edge_mask = None
+        objects = []
 
         for k in range(len(image_pos)):
             x = image_pos[k].x
@@ -634,111 +618,55 @@ class InputFiles(Input):
                 bounds = bounds & image.bounds
                 if not bounds.isDefined():
                     logger.warning("Star at position %f,%f is off the edge of the image.", x, y)
-                    logger.warning("Skipping this star.")
+                    logger.warning("Skipping this object.")
                     continue
                 if use_partial:
                     logger.info("Star at position %f,%f overlaps the edge of the image.  "
                                 "Using smaller than the full stamp size: %s", x, y, bounds)
                 else:
                     logger.warning("Star at position %f,%f overlaps the edge of the image.", x, y)
-                    logger.warning("Skipping this star.")
+                    logger.warning("Skipping this object.")
                     continue
             stamp = image[bounds].copy()
             wt_stamp = wt[bounds].copy()
             props = { 'chipnum' : chipnum,
-                      'gain' : gain[k],
                     }
 
-            # if a star is totally masked, then don't add it!
+            # if an object is totally masked, then don't add it!
             if np.all(wt_stamp.array == 0):
                 logger.warning("Star at position %f,%f is completely masked.", x, y)
-                logger.warning("Skipping this star.")
+                logger.warning("Skipping this object.")
                 continue
 
+            # Add this object's entry in each list of the extra_props dictionary
+            # to the StarData properties dictionary
+            for key in extra_props:
+                logger.debug("Assigning {} value = {}".format(key, extra_props[key][k]))
+                props[key] = extra_props[key][k]
+
             # If any pixels are saturated, skip it.
             max_val = np.max(stamp.array)
-            if satur is not None and max_val > satur:
+            if max_val > props.get('satur', np.inf):
                 logger.warning("Star at position %f,%f has saturated pixels.", x, y)
                 logger.warning("Maximum value is %f.", max_val)
-                logger.warning("Skipping this star.")
+                logger.warning("Skipping this object.")
                 continue
 
-            # here we remove stars that have been at least partially covered by a mask
-            # and thus have weight exactly 0 in at least a certain number of pixels of their
-            # postage stamp
-            if max_mask_pixels is not None:
-                n_masked = np.prod(wt_stamp.array.shape) - np.count_nonzero(wt_stamp.array)
-                if n_masked >= max_mask_pixels:
-                    logger.warning("Star at position %f,%f has %i masked pixels, ", x, y, n_masked)
-                    logger.warning("Skipping this star.")
-                    continue
-                
             # Subtract the sky
-            if sky is not None:
-                logger.debug("Subtracting off sky = %f", sky[k])
+            if 'sky' in props:
+                sky = props['sky']
+                logger.debug("Subtracting off sky = %f", sky)
                 logger.debug("Median pixel value = %f", np.median(stamp.array))
-                stamp -= sky[k]
-                props['sky'] = sky[k]
-
-            # Check the snr and limit it if appropriate
-            snr = calculateSNR(stamp, wt_stamp)
-            logger.debug("SNR = %f",snr)
-            if min_snr is not None and snr < min_snr:
-                logger.info("Skipping star at position %f,%f with snr=%f."%(x,y,snr))
-                continue
-            if max_snr > 0 and snr > max_snr:
-                factor = (max_snr / snr)**2
-                logger.debug("Scaling noise by factor of %f to achieve snr=%f", factor, max_snr)
-                wt_stamp *= factor
-                snr = max_snr
-            props['snr'] = snr
+                stamp -= sky
 
             pos = galsim.PositionD(x,y)
             data = StarData(stamp, pos, weight=wt_stamp, pointing=pointing,
                             properties=props)
             star = Star(data, None)
-            g = gain[k]
-            if g is not None:
-                logger.debug("Adding Poisson noise to weight map according to gain=%f",g)
-                star = star.addPoisson(gain=g)
 
+            objects.append(star)
 
-            if max_edge_frac is not None and max_edge_frac < 1:
-                flux = np.sum(star.image.array)
-                try:
-                    flux_extra = np.sum(star.image.array[edge_mask])
-                    flux_frac = flux_extra / flux
-                except IndexError:
-                    logger.warning("Star at position %f,%f overlaps the edge of the image and "+
-                                   "max_edge_frac cut is set.", x, y)
-                    logger.warning("Skipping this star.")
-                    continue
-                if flux_frac > max_edge_frac:
-                    logger.warning("Star at position %f,%f fraction of flux near edge of stamp "+
-                                   "exceeds cut: %f > %f", x, y, flux_frac, max_edge_frac)
-                    logger.warning("Skipping this star.")
-                    continue
-
-            stars.append(star)
-            nstars_in_image += 1
-
-        if hsm_size_reject != 0:
-            # Calculate the hsm size for each star and throw out extreme outliers.
-            sigma = [star.hsm[3] for star in stars]
-            med_sigma = np.median(sigma)
-            iqr_sigma = scipy.stats.iqr(sigma)
-            logger.debug("Doing hsm sigma rejection.")
-            while np.max(np.abs(sigma - med_sigma)) > hsm_size_reject * iqr_sigma:
-                logger.debug("median = %s, iqr = %s, max_diff = %s",
-                                med_sigma, iqr_sigma, np.max(np.abs(sigma-med_sigma)))
-                k = np.argmax(np.abs(sigma-med_sigma))
-                logger.debug("remove k=%d: sigma = %s, pos = %s",k,sigma[k],stars[k].image_pos)
-                del sigma[k]
-                del stars[k]
-                med_sigma = np.median(sigma)
-                iqr_sigma = scipy.stats.iqr(sigma)
-
-        return stars
+        return objects
 
     def setWCS(self, config, logger):
         self.wcs_list = []
@@ -786,13 +714,17 @@ class InputFiles(Input):
         return newweight, gain
 
     @staticmethod
-    def readImage(image_file_name, image_hdu, weight_hdu, badpix_hdu, noise, logger):
+    def readImage(image_file_name, image_hdu, weight_hdu, badpix_hdu,
+                  sky_file_name, sky_hdu, noise, logger):
         """Read in the image and weight map (or make one if no weight information is given
 
         :param image_file_name: The name of the file to read.
         :param image_hdu:       The hdu of the main image.
         :param weight_hdu:      The hdu of the weight image (if any).
         :param badpix_hdu:      The hdu of the bad pixel mask (if any).
+        :param sky_file_name:   A file to use for a sky background to subtract from the image
+                                (if any).
+        :param sky_hdu:         The hdu to use in the sky_file_name (if any).
         :param noise:           A constant noise value to use in lieu of a weight map.
         :param logger:          A logger object for logging debug info.
 
@@ -802,6 +734,13 @@ class InputFiles(Input):
         logger.warning("Reading image file %s",image_file_name)
         image = galsim.fits.read(image_file_name, hdu=image_hdu)
 
+        # If requested, subtract a sky image
+        if sky_file_name is not None:
+            hdu_str = " from hdu %s"%(sky_hdu) if sky_hdu is not None else ""
+            logger.info("Reading sky image %s.", sky_file_name + hdu_str)
+            sky = galsim.fits.read(sky_file_name, hdu=sky_hdu)
+            image -= sky
+
         # Either read in the weight image, or build a dummy one
         if weight_hdu is not None:
             logger.info("Reading weight image from hdu %d.", weight_hdu)
@@ -840,6 +779,7 @@ class InputFiles(Input):
                 logger.error("According to the bad pixel array in %s, all pixels are masked!",
                              image_file_name)
             weight.array[badpix.array != 0] = 0
+
         return image, weight
 
     @staticmethod
@@ -860,7 +800,7 @@ class InputFiles(Input):
     @staticmethod
     def readStarCatalog(cat_file_name, cat_hdu, x_col, y_col,
                         ra_col, dec_col, ra_units, dec_units, image,
-                        flag_col, skip_flag, use_flag, sky_col, gain_col,
+                        flag_col, skip_flag, use_flag, property_cols, sky_col, gain_col,
                         sky, gain, satur, nstars, image_file_name, stamp_size, logger):
         """Read in the star catalogs and return lists of positions for each star in each image.
 
@@ -878,6 +818,7 @@ class InputFiles(Input):
                                 Items with flag & skip_flag != 0 will be skipped.
         :param use_flag:        The flag indicating which items to use. [default: None]
                                 Items with flag & use_flag == 0 will be skipped.
+        :param property_cols:   A list of column names with star properties (e.g. star colors).
         :param sky_col:         A column with sky (background) levels.
         :param gain_col:        A column with gain values.
         :param sky:             Either a float value for the sky to use for all objects or a str
@@ -891,10 +832,15 @@ class InputFiles(Input):
         :param stamp_size:      The stamp size being used for the star stamps.
         :param logger:          A logger object for logging debug info. [default: None]
 
-        :returns: lists image_pos, sky, gain, satur
+        :returns: lists image_pos, extra_props
         """
         import fitsio
 
+        if cat_file_name is None:
+            # This is possible e.g. when loading images into an existing list of star instances.
+            logger.warning("Not reading star catalog.")
+            return None, None
+
         # Read in the star catalog
         logger.warning("Reading star catalog %s.",cat_file_name)
         cat = fitsio.read(cat_file_name, cat_hdu)
@@ -992,11 +938,25 @@ class InputFiles(Input):
         image_pos = [ pos for pos in image_pos if big_bounds.includes(pos) ]
         logger.debug("After remove those that are off the image, len = %s",len(image_pos))
 
+        # Make a dictionary of the star properties:
+        extra_props = {}
+        if property_cols is not None:
+            logger.debug('property_cols = %s'%property_cols)
+            for col_name in property_cols:
+                if col_name not in cat.dtype.names:
+                    raise ValueError("Entry in property_cols = " +
+                                     "%s is not a column in %s"%(col_name,cat_file_name))
+                extra_props[col_name] = cat[col_name]
+
+        # If we used a flag column, keep it as a property.
+        if flag_col is not None:
+            extra_props[flag_col] = cat[flag_col]
+
         # Make the list of sky values:
         if sky_col is not None:
             if sky_col not in cat.dtype.names:
                 raise ValueError("sky_col = %s is not a column in %s"%(sky_col,cat_file_name))
-            sky = cat[sky_col]
+            extra_props['sky'] = cat[sky_col]
         elif sky is not None:
             try:
                 sky = float(sky)
@@ -1007,9 +967,7 @@ class InputFiles(Input):
                 if sky not in header:
                     raise KeyError("Key %s not found in FITS header"%sky)
                 sky = float(header[sky])
-            sky = np.array([sky]*len(cat), dtype=float)
-        else:
-            sky = None
+            extra_props['sky'] = np.array([sky]*len(cat), dtype=float)
 
         # Make the list of gain values:
         # TODO: SV and Y1 DES images have two gain values, GAINA, GAINB.  It would be nice if we
@@ -1018,7 +976,7 @@ class InputFiles(Input):
         if gain_col is not None:
             if gain_col not in cat.dtype.names:
                 raise ValueError("gain_col = %s is not a column in %s"%(gain_col,cat_file_name))
-            gain = cat[gain_col]
+            extra_props['gain'] = cat[gain_col]
         elif gain is not None:
             try:
                 gain = float(gain)
@@ -1029,9 +987,9 @@ class InputFiles(Input):
                 if gain not in header:
                     raise KeyError("Key %s not found in FITS header"%gain)
                 gain = float(header[gain])
-            gain = np.array([gain]*len(cat), dtype=float)
+            extra_props['gain'] = np.array([gain]*len(cat), dtype=float)
         else:
-            gain = [None] * len(cat)
+            extra_props['gain'] = [None] * len(cat)
 
         # Get the saturation level
         if satur is not None:
@@ -1046,8 +1004,11 @@ class InputFiles(Input):
                     raise KeyError("Key %s not found in FITS header"%satur)
                 satur = float(header[satur])
                 logger.debug("Using saturation from header: %s",satur)
+            extra_props['satur'] = np.array([satur]*len(cat), dtype=float)
+        else:
+            extra_props['satur'] = np.array([np.inf]*len(cat), dtype=float)
 
-        return image_pos, sky, gain, satur
+        return image_pos, extra_props
 
     def setPointing(self, ra, dec, logger=None):
         """Set the pointing attribute based on the input ra, dec (given in the initializer)
@@ -1128,7 +1089,3 @@ class InputFiles(Input):
             dec = header[dec]
             # Recurse to do further parsing.
             self.setPointing(ra, dec, logger)
-
-# Workaround for python 2.7, which can't directly call staticmethods in multiprocessing.
-def call_makeStarsFromImage(cls, *args, **kwargs):
-    return cls._makeStarsFromImage(*args, **kwargs)
