@@ -98,11 +98,16 @@ class SizeMagStats(Stats):
         fig = Figure(figsize=(8,6))
         ax = fig.add_subplot(1,1,1)
 
-        xmin = np.floor(np.min(self.m_star))
-        xmin = np.min(self.m_init, initial=xmin)  # Do it this way in case m_init is empty.
-        xmax = np.ceil(np.max(self.m_star))
-        xmax = np.max(self.m_obj, initial=xmax)   # Likewise, m_obj might be empty.
-        ymax = max(np.median(self.T_star)*2, np.max(self.T_star)*1.01)
+        if len(self.m_star) == 0:
+            xmin = np.floor(np.min(self.m_obj, initial=0))
+            xmax = np.ceil(np.max(self.m_obj, initial=100))
+            ymax = np.median(self.T_obj)*2 if len(self.T_obj) > 0 else 1.0
+        else:
+            xmin = np.floor(np.min(self.m_star))
+            xmin = np.min(self.m_init, initial=xmin)  # Do it this way in case m_init is empty.
+            xmax = np.ceil(np.max(self.m_star))
+            xmax = np.max(self.m_obj, initial=xmax)   # Likewise, m_obj might be empty.
+            ymax = max(np.median(self.T_star)*2, np.max(self.T_star)*1.01)
         ax.set_xlim(xmin, xmax)
         ax.set_ylim(0, ymax)
         ax.set_xlabel('Magnitude (ZP=%s)'%self.zeropoint, fontsize=15)
@@ -131,11 +136,11 @@ class SizeMagStats(Stats):
                              color='black', marker='o', s=2., **kwargs)
             init = ax.scatter(m[s][g[s] == 2], T[s][g[s] == 2],
                               color='magenta', marker='o', s=8., **kwargs)
-            psf = ax.scatter(m[s][g[s] == 3], T[s][g[s] == 3],
+            star = ax.scatter(m[s][g[s] == 3], T[s][g[s] == 3],
+                              color='green', marker='*', s=40., **kwargs)
+            psf = ax.scatter(m[s][g[s] == 4], T[s][g[s] == 4],
                              marker='o', s=50.,
                              facecolors='none', edgecolors='cyan', **kwargs)
-            star = ax.scatter(m[s][g[s] == 4], T[s][g[s] == 4],
-                              color='green', marker='*', s=40., **kwargs)
 
         ax.legend([obj, init, star, psf],
               ['Detected Object', 'Candidate Star', 'PSF Star', 'PSF Model'],
@@ -200,7 +205,7 @@ class SmallBrightSelect(Select):
         """
         logger = galsim.config.LoggerWrapper(logger)
 
-        logger.info("Selecting small/bright objects as stars")
+        logger.warning("Selecting small/bright objects as stars")
 
         logger.debug("Initial count = %s", len(objects))
 
@@ -215,7 +220,7 @@ class SmallBrightSelect(Select):
         obj_shapes = np.array([ obj.hsm for obj in objects ])
         flag_obj = obj_shapes[:, 6]
         f_obj = obj_shapes[:, 0]
-        T_obj = obj_shapes[:, 3]
+        T_obj = 2*obj_shapes[:, 3]**2
 
         # Getting rid of the flags will mess with the indexing, so keep track of the original
         # index numbers.
@@ -304,8 +309,9 @@ class SmallBrightSelect(Select):
         select_index = orig_index[select]
         logger.debug("select_index = %s",select_index)
         stars = [objects[i] for i in select_index]
-        logger.debug("sizes of stars = %s",[s.hsm[3] for s in stars])
+        logger.debug("sizes of stars = %s",[2*s.hsm[3]**2 for s in stars])
         logger.debug("fluxs of stars = %s",[s.hsm[0] for s in stars])
+        logger.warning("Bright/small selection found %d likely stars",len(stars))
 
         return stars
 
@@ -390,9 +396,9 @@ class SizeMagSelect(Select):
         """
         logger = galsim.config.LoggerWrapper(logger)
 
-        logger.info("Selecting stars according to locus in size-magnitude diagram")
+        logger.warning("Selecting stars according to locus in size-magnitude diagram")
 
-        stars = Select.process(self.initial_select, objects, logger=logger)
+        stars = Select.process(self.initial_select, objects, logger=logger, select_only=True)
 
         logger.debug("N objects = %s", len(objects))
         logger.debug("N initial stars = %s", len(stars))
@@ -401,7 +407,7 @@ class SizeMagSelect(Select):
         obj_shapes = np.array([ obj.hsm for obj in objects ])
         flag_obj = obj_shapes[:, 6]
         f_obj = obj_shapes[:, 0]
-        T_obj = obj_shapes[:, 3]
+        T_obj = 2*obj_shapes[:, 3]**2
         u_obj = np.array([ obj.u for obj in objects ])
         v_obj = np.array([ obj.v for obj in objects ])
 
@@ -433,7 +439,7 @@ class SizeMagSelect(Select):
         star_shapes = np.array([ star.hsm for star in stars ])
         mask = star_shapes[:, 6] == 0
         logf_star = np.log(star_shapes[mask, 0])
-        logT_star = np.log(star_shapes[mask, 3])
+        logT_star = np.log(2*star_shapes[mask, 3]**2)
         u_star = np.array([ star.u for star in stars ])[mask]
         v_star = np.array([ star.v for star in stars ])[mask]
         logger.debug("logf_star = %s",logf_star)
@@ -525,14 +531,16 @@ class SizeMagSelect(Select):
             logT_star = logT_obj[select]
             u_star = u_obj[select]
             v_star = v_obj[select]
-            logger.info("SizeMag iteration %d => N stars = %d", i_iter, len(logf_star))
-            logger.info("Mean logT of stars = %s, std = %s", np.mean(logT_star), np.std(logT_star))
+            logger.warning("SizeMag iteration %d => N stars = %d", i_iter, len(logf_star))
+            logger.warning("Mean logT of stars = %s, std = %s", np.mean(logT_star), np.std(logT_star))
 
         select_index = orig_index[select]
         logger.debug("select_index = %s",select_index)
         stars = [objects[i] for i in select_index]
-        logger.debug("sizes of stars = %s",[s.hsm[3] for s in stars])
+        logger.debug("sizes of stars = %s",[2*s.hsm[3]**2 for s in stars])
         logger.debug("fluxs of stars = %s",[s.hsm[0] for s in stars])
+        logger.warning("SizeMag selection found %d likely stars",len(stars))
+
         return stars
 
     @staticmethod
