@@ -70,7 +70,7 @@ class Star(object):
         star.flux       The flux of the object
         star.center     The nominal center of the object (not necessarily the centroid)
         star.is_reserve Whether the star is reserved from being used to fit the PSF
-        star.hsm        HSM measurements for this star as a tuple: (flux, cenu, cenv, size, g1, g2)
+        star.hsm        HSM measurements for this star as a tuple: (flux, cenu, cenv, sigma, g1, g2)
     """
     def __init__(self, data, fit):
         """Constructor for Star instance.
@@ -165,7 +165,7 @@ class Star(object):
         This usually isn't called directly.  The results are accessible as star.hsm,
         which caches the results, so repeated access is efficient.
 
-        :returns: (flux, cenu, cenv, size, g1, g2, flag)
+        :returns: (flux, cenu, cenv, sigma, g1, g2, flag)
         """
         image, weight, image_pos = self.data.getImage()
         # Note that FindAdaptiveMom only respects the weight function in a binary sense.
@@ -192,7 +192,15 @@ class Star(object):
 
         localwcs = image.wcs.local(image_pos)
         center = localwcs.toWorld(mom.moments_centroid) - localwcs.toWorld(image_pos)
+
+        # Do a few sanity checks and flag likely bad fits.
         flag = mom.moments_status
+        if flag != 0:
+            flag = 1
+        if flux < 0:
+            flag |= 2
+        if center.x**2 + center.y**2 > 1:
+            flag |= 4
 
         return flux, center.x, center.y, sigma, shape.g1, shape.g2, flag
 
