@@ -50,7 +50,7 @@ def make_gaussian_data(sigma, u0, v0, flux, noise=0., du=1., fpu=0., fpv=0., nsi
     star.image.setOrigin(0,0)
     star.weight.setOrigin(0,0)
 
-    g.drawImage(star.image, method='no_pixel', use_true_center=False,
+    g.drawImage(star.image, use_true_center=False,
                 offset=galsim.PositionD(nom_u0/du,nom_v0/du))
 
     if noise != 0:
@@ -84,7 +84,9 @@ def test_simplest():
     flux1 = star.fit.flux
 
     # It doesn't get any better after another iteration.
-    star = mod.fit(star)
+    # Also check a trivial convert_func that it does the same thing.
+    convert_func = lambda prof: prof
+    star = mod.fit(star, convert_func=convert_func)
     star = mod.reflux(star)
     print('Flux after fit 2:',star.fit.flux)
     np.testing.assert_almost_equal(star.fit.flux/influx, 1.0, decimal=3)
@@ -508,11 +510,11 @@ def test_undersamp():
     else:
         size = 15
 
-    # Pixelized model with Lanczos 3 interpolation, slightly smaller than data
-    # than the data
+    # Pixelized model with Lanczos 3 interpolation
+    # Model pixels are about 2/3 the size of the real pixels.
     pixinterp = piff.Lanczos(3)
     du = 0.5
-    mod = piff.PixelGrid(0.25, size, pixinterp)
+    mod = piff.PixelGrid(0.33, size, pixinterp)
 
     # Interpolator will be constant
     interp = piff.Polynomial(order=0)
@@ -542,6 +544,50 @@ def test_undersamp():
     # Polynomial doesn't need this, but it should work nonetheless.
     interp.initialize(stars)
 
+    # Work on one star alone first
+    b = galsim.BoundsI(16-size//3,16+size//3,16-size//3,16+size//3)
+    s1 = mod.draw(s0)
+    chisq1 = np.sum((s0.image[b].array - s1.image[b].array)**2*s0.weight[b].array)
+    print('chisq after initialize = ',chisq1)
+    print('nominal chisq = ',s1.fit.chisq)
+    assert chisq1 > 5.e5  # Pretty large at the start before fitting.
+
+    s2 = mod.fit(s0)
+    image = mod.draw(s2).image
+    chisq2 = np.sum((s0.image[b].array - image[b].array)**2*s0.weight[b].array)
+    print('chisq after fit = ',chisq2)
+    print('nominal chisq = ',s2.fit.chisq)
+    # It's not zero yet because of the flux flexibility, so the sum doesn't end up equal to 1.
+    assert chisq2 < 5.e4
+    # The nominal thinks it got it right though.
+    assert s2.fit.chisq < 1.e-6
+
+    s3 = mod.reflux(s2)
+    image = mod.draw(s3).image
+    chisq3 = np.sum((s0.image[b].array - image[b].array)**2*s0.weight[b].array)
+    print('chisq after reflux = ',chisq3)
+    print('nominal chisq = ',s3.fit.chisq)
+    # After refluxing, the chisq over the bounds where we are fitting is much better.
+    assert chisq3 < 1.0
+    # Over the full image, it's still not great, but this is expected, since the pixelgrid
+    # is only really fitting the central area.
+    assert s3.fit.chisq < 1.e6
+
+    s4 = mod.fit(s3)
+    image = mod.draw(s4).image
+    chisq = np.sum((s0.image[b].array - image[b].array)**2*s0.weight[b].array)
+    print('chisq after fit = ',chisq)
+    print('nominal chisq = ',s4.fit.chisq)
+    # Fitting again doesn't mess it up, but it's not any better.
+    assert chisq < 1.0
+
+    s5 = mod.reflux(s4)
+    image = mod.draw(s5).image
+    chisq = np.sum((s0.image[b].array - image[b].array)**2*s0.weight[b].array)
+    print('chisq after reflux = ',chisq)
+    print('nominal chisq = ',s5.fit.chisq)
+    assert chisq < 1.0
+
     oldchisq = 0.
     # Iterate solution using interpolator
     for iteration in range(1):
@@ -795,7 +841,8 @@ def test_single_image():
     sky_level = 1000
     noise_sigma = 0.1  # Not much noise to keep this an easy test.
     image += sky_level
-    image.addNoise(galsim.GaussianNoise(sigma=noise_sigma))
+    rng = galsim.BaseDeviate(1234)
+    image.addNoise(galsim.GaussianNoise(sigma=noise_sigma, rng=rng))
 
     # Write out the image to a file
     image_file = os.path.join('output','pixel_moffat_image.fits')
@@ -824,7 +871,7 @@ def test_single_image():
 
     # Check image
     assert input.nimages == 1
-    image1, _, image_pos, _, _, _ = input.getRawImageData(0)
+    image1, _, image_pos, _ = input.getRawImageData(0)
     np.testing.assert_equal(image1.array, image.array)
 
     # Check catalog
@@ -859,7 +906,7 @@ def test_single_image():
         order = 1
 
     # These tests are slow, and it's really just doing the same thing three times, so
-    # only do the first one when running via nosetests.
+    # only do the second one when running via nosetests.
     psf_file = os.path.join('output','pixel_psf.fits')
     if __name__ == '__main__':
         # Process the star data
@@ -1009,8 +1056,6 @@ def test_des_image():
             'ra' : 'TELRA',
             'dec' : 'TELDEC',
             'gain' : 'GAINA',
-            'reserve_frac' : 0.2,
-            'seed' : 1234,
             # Test explicitly specifying the wcs (although it is the same here as what is in the
             # image anyway).
             'wcs' : {
@@ -1018,6 +1063,10 @@ def test_des_image():
                 'file_name': image_file
             },
         },
+        'select' : {
+            'reserve_frac' : 0.2,
+            'seed' : 1234,
+        },
         'output' : {
             'file_name' : psf_file,
         },
@@ -1111,7 +1160,8 @@ def test_des_image():
             offset = s.center_to_offset(s.fit.center)
             image = psf.draw(x=s['x'], y=s['y'], stamp_size=stamp_size,
                              flux=s.fit.flux, offset=offset)
-            np.testing.assert_allclose(image.array, fit_stamp.array, rtol=1.e-6, atol=1.e-4)
+            np.testing.assert_allclose(image.array, fit_stamp.array, rtol=1.e-6,
+                                       atol=1.e-6 * s.fit.flux)
 
         print('n_good, marginal, bad = ',n_good,n_marginal,n_bad)
         # The real counts are 10 and 2.  So this says make sure any updates to the code don't make
@@ -1233,10 +1283,11 @@ def test_des2():
             'dec' : 'TELDEC',
             'gain' : 1.0,
             # Real version uses pixmappy WCS, but that's not important for this test, so skip it
-
+            'stamp_size' : 25
+        },
+        'select' : {
             'max_snr' : 100,
             'min_snr' : 20,
-            'stamp_size' : 25
         },
         'output' : {
             'file_name' : psf_file,
@@ -1307,8 +1358,8 @@ def test_des2():
     test_stamp = psf.draw(x=test_x, y=test_y, stamp_size=25)
     np.testing.assert_allclose(test_stamp.array, check_stamp.array, rtol=tol, atol=tol)
 
-    # With only 10 stars (or fewer), there are not enough constraints for a 3rd order solution
-    config['input']['nstars'] = 10
+    # With only 9 stars (or fewer), there are not enough constraints for a 3rd order solution
+    config['input']['nstars'] = 9
     config['psf']['interp']['order'] = 3
     with np.testing.assert_raises(RuntimeError):
         piff.piffify(config)
@@ -1381,6 +1432,194 @@ def test_var():
     print('mean ratio = ',np.mean(star.fit.params_var/var))
     np.testing.assert_allclose(star.fit.params_var, var, rtol=0.2)
 
+@timer
+def test_color():
+    """Test having a color-dependent size.
+    """
+    pixel_scale = 0.3
+
+    image = galsim.ImageF(1024,1024, scale=pixel_scale)
+
+    # Draw stars in a grid so we know they don't overlap.
+    x_list = []
+    y_list = []
+    color_list = []
+    np_rng = np.random.RandomState(1234)
+    rng = galsim.UniformDeviate(1234)
+    for x in range(50,1000,100):
+        for y in range(50,1000,100):
+            # color is random between -1 and +1.
+            color = np_rng.uniform(-1,1)
+            # Add a +- 1 pixel offset to nominal position.
+            xcen = x + np_rng.uniform(-1,1)
+            ycen = y + np_rng.uniform(-1,1)
+            sigma = 1.0 + 0.1 * color
+            influx = 150.
+            star = galsim.Gaussian(sigma=sigma, flux=influx)
+            b = galsim.BoundsI(x-16,x+16,y-16,y+16)
+            star.drawImage(image=image[b], center=galsim.PositionD(xcen,ycen))
+            x_list.append(xcen)
+            y_list.append(ycen)
+            color_list.append(color)
+
+    noise_sigma = 0.1
+    image.addNoise(galsim.GaussianNoise(sigma=noise_sigma, rng=rng))
+
+    # Write out the image and catalog
+    image_file = os.path.join('output','pixel_color_image.fits')
+    image.write(image_file)
+    dtype = [ ('x','f8'), ('y','f8'), ('color','f8') ]
+    data = np.empty(len(x_list), dtype=dtype)
+    data['x'] = x_list
+    data['y'] = y_list
+    data['color'] = color_list
+    cat_file = os.path.join('output','pixel_color_cat.fits')
+    fitsio.write(cat_file, data, clobber=True)
+
+    # Use InputFiles to read these back in
+    config = { 'image_file_name': image_file,
+               'cat_file_name': cat_file,
+               'stamp_size': 32,
+               'noise' : noise_sigma**2,
+               'property_cols' : ['color']
+             }
+    input = piff.InputFiles(config)
+    assert input.image_file_name == [image_file]
+    assert input.cat_file_name == [cat_file]
+    assert input.nimages == 1
+    image1, _, image_pos, props = input.getRawImageData(0)
+    np.testing.assert_equal(image1.array, image.array)
+    assert len(image_pos) == len(x_list)
+    np.testing.assert_equal([pos.x for pos in image_pos], x_list)
+    np.testing.assert_equal([pos.y for pos in image_pos], y_list)
+    np.testing.assert_equal(props['color'], color_list)
+
+    # Make stars
+    orig_stars = input.makeStars()
+    assert len(orig_stars) == len(x_list)
+    assert orig_stars[0].image.array.shape == (32,32)
+
+    if __name__ == '__main__':
+        logger = piff.config.setup_logger(2)
+    else:
+        #logger = piff.config.setup_logger(3)
+        logger = None
+
+    psf_file = os.path.join('output','pixel_color.fits')
+
+    # 2nd order in u,v, 1st order in color
+    size = 15
+    mod = piff.PixelGrid(pixel_scale, size)
+    interp = piff.BasisPolynomial(order=[2,2,1], keys=['u','v','color'])
+    psf = piff.SimplePSF(mod, interp)
+    psf.fit(orig_stars, {0:image1.wcs}, None, logger=logger)
+
+    Tstar_list = []
+    Tpsf_list = []
+    dT_list = []
+    for s in psf.stars:
+        fitted = psf.drawStar(s)
+        fit_stamp = fitted.image
+        orig_stamp = s.image
+        weight = s.weight
+        resid = fit_stamp - orig_stamp
+        chisq = np.sum(resid.array**2 * weight.array)
+        dof = np.sum(weight.array != 0)
+        print('color = ',s['color'],'chisq = ',chisq,'dof = ',dof)
+        assert chisq < dof * 1.5  # In fact, chisq < dof in all cases here.
+
+        # Check the convenience function that an end user would typically use
+        offset = s.center_to_offset(s.fit.center)
+        image = psf.draw(x=s['x'], y=s['y'], color=s['color'],
+                         stamp_size=32, flux=s.fit.flux, offset=offset)
+        # They may be up to 1 pixel off in either direction, so find the common bounds.
+        b = image.bounds & fit_stamp.bounds
+        np.testing.assert_allclose(image[b].array, fit_stamp[b].array, rtol=1.e-6, atol=1.e-4)
+
+        # Measure the size of star and psf
+        Tstar = orig_stamp.FindAdaptiveMom(weight=weight, strict=False).moments_sigma**2 * 2
+        Tpsf = fit_stamp.FindAdaptiveMom(weight=weight, strict=False).moments_sigma**2 * 2
+        Tstar_list.append(Tstar)
+        Tpsf_list.append(Tpsf)
+        dT_list.append(Tpsf - Tstar)
+
+    # An error if color is not provided.
+    with np.testing.assert_raises(TypeError):
+        image = psf.draw(x=s['x'], y=s['y'], stamp_size=32, flux=s.fit.flux, offset=offset)
+    # Also error if extra or wrong color parameter
+    with np.testing.assert_raises(TypeError):
+        image = psf.draw(x=s['x'], y=s['y'], color_ri=s['color'],
+        stamp_size=32, flux=s.fit.flux, offset=offset)
+    with np.testing.assert_raises(TypeError):
+        image = psf.draw(x=s['x'], y=s['y'], color=s['color'], color_ri=0.5,
+        stamp_size=32, flux=s.fit.flux, offset=offset)
+
+    m, b = np.polyfit(color_list, Tstar_list, deg=1)
+    print(f'Tstar vs color: m = {m}, b = {b}')
+    # Expected slope = 0.1  (explicit coefficient in sigma vs color)
+    #                  * 2  (because sigma**2)
+    #                  * 2  (because T = 2 * sigma**2)
+    #                  /0.3**2  (because T was measured in pixels, not arcsec)
+    #                = 4.45
+    # Check fits of size vs color
+    assert np.isclose(m, 4.45, rtol=0.01)  # This is very close
+    m, b = np.polyfit(color_list, Tpsf_list, deg=1)
+    print(f'Tpsf vs color: m = {m}, b = {b}')
+    assert np.isclose(m, 4.45, rtol=0.2)   # This has noise, so only approximate
+    m, b = np.polyfit(color_list, dT_list, deg=1)
+    print(f'dT vs color: m = {m}, b = {b}')
+    assert np.isclose(m, 0., atol=1)   # The noise in Tpsf here is still here as atol.
+
+    # Do the whole thing with the config parser
+    psf_file = os.path.join('output','pixel_color.piff')
+    config = {
+        'input' : {
+            'image_file_name' : image_file,
+            'cat_file_name' : cat_file,
+            'x_col' : 'x',
+            'y_col' : 'y',
+            'property_cols' : ['color'],
+            'noise' : noise_sigma**2,
+            'stamp_size' : 32,
+        },
+        'output' : {
+            'file_name' : psf_file
+        },
+        'psf' : {
+            'model' : {
+                'type' : 'PixelGrid',
+                'scale' : pixel_scale,
+                'size' : size,
+            },
+            'interp' : {
+                'type' : 'BasisPolynomial',
+                'order' : [2, 2, 1],
+                'keys': ['u', 'v', 'color'],
+            },
+        },
+    }
+    if __name__ == '__main__':
+        config['verbose'] = 2
+    else:
+        config['verbose'] = 0
+
+    print("Running piffify function")
+    piff.piffify(config)
+    psf = piff.read(psf_file)
+
+    for s in psf.stars:
+        orig_stamp = s.image
+        weight = s.weight
+        offset = s.center_to_offset(s.fit.center)
+        image = psf.draw(x=s['x'], y=s['y'], color=s['color'],
+                         stamp_size=32, flux=s.fit.flux, offset=offset)
+        resid = image - orig_stamp
+        chisq = np.sum(resid.array**2 * weight.array)
+        dof = np.sum(weight.array != 0)
+        print('color = ',s['color'],'chisq = ',chisq,'dof = ',dof)
+        assert chisq < dof * 1.5  # These chisq are even (a lot) smaller.  Not sure why...
+                                  # Anyway, I think the fit is working, just this test doesn't
+                                  # seem quite the right thing.
 
 if __name__ == '__main__':
     #import cProfile, pstats
@@ -1400,6 +1639,7 @@ if __name__ == '__main__':
     test_des_image()
     test_des2()
     test_var()
+    test_color()
     #pr.disable()
     #ps = pstats.Stats(pr).sort_stats('tottime')
     #ps.print_stats(20)
