@@ -186,7 +186,7 @@ def test_single_image():
 
     # Check image
     assert input.nimages == 1
-    image1, _, image_pos, _, _, _ = input.getRawImageData(0)
+    image1, _, image_pos, _ = input.getRawImageData(0)
     np.testing.assert_equal(image1.array, image.array)
 
     # Check catalog
@@ -202,7 +202,7 @@ def test_single_image():
                'stamp_size': 48 }
     input = piff.InputFiles(config, logger=logger)
     assert input.nimages == 1
-    _, _, image_pos, _, _, _ = input.getRawImageData(0)
+    _, _, image_pos, _ = input.getRawImageData(0)
     assert len(image_pos) == 7
 
     # Make star data
@@ -241,7 +241,7 @@ def test_single_image():
     assert psf.chisq_thresh == 0.1
     assert psf.max_iter == 30
     assert psf.outliers == None
-    assert psf.extra_interp_properties == []
+    assert psf.interp_property_names == ('u','v')
 
     # Now test running it via the config parser
     psf_file = os.path.join('output','simple_psf.fits')
@@ -272,6 +272,11 @@ def test_single_image():
     assert psf.chisq_thresh == 0.2
     assert psf.max_iter == 10
 
+    # Error if input has no stars
+    with np.testing.assert_raises(RuntimeError):
+        psf.fit([], wcs, pointing, logger=logger)
+
+    # Do the fit
     psf.fit(orig_stars, wcs, pointing, logger=logger)
     test_star = psf.interp.interpolate(target)
     np.testing.assert_almost_equal(test_star.fit.params, true_params, decimal=4)
@@ -298,6 +303,10 @@ def test_single_image():
         assert test_star_nocopy.image.array[1,1] == target_star_copy.image.array[1,1]
         assert test_star_copy.image.array[1,1] == target_star_copy.image.array[1,1]
 
+        test_star_center = draw(test_star_copy, copy_image=True, center=(x+1,y+1))
+        np.testing.assert_almost_equal(test_star_center.image.array[1:,1:],
+                                       test_star_copy.image.array[:-1,:-1])
+
     # test that draw works
     test_image = psf.draw(x=target['x'], y=target['y'], stamp_size=config['input']['stamp_size'],
                           flux=target.fit.flux, offset=target.fit.center)
@@ -476,9 +485,11 @@ def test_reserve():
         'input' : {
             'image_file_name' : image_file,
             'cat_file_name' : cat_file,
-            'reserve_frac' : 0.2,
             'stamp_size' : 32,
         },
+        'select' : {
+            'reserve_frac' : 0.2,
+        },
         'psf' : {
             'model' : { 'type' : 'Gaussian',
                         'fastfit': True,
@@ -499,12 +510,27 @@ def test_reserve():
     print('reserve = %s/%s'%(nreserve,ntot))
     assert nreserve == 2
     assert ntot == 10
-    print('dof =? ',(32*32 - 6) * (ntot-nreserve))
-    assert psf.dof == (32*32 - 6) * (ntot-nreserve)
+    print('dof =? ',(32*32 - 3) * (ntot-nreserve))
+    assert psf.dof == (32*32 - 3) * (ntot-nreserve)
     for star in psf.stars:
         # Fits should be good for both reserve and non-reserve stars
         np.testing.assert_almost_equal(star.fit.params, true_params, decimal=4)
 
+    # Check the old API of putting reserve_frac in input
+    config['input']['reserve_frac'] = 0.2
+    del config['select']
+    with CaptureLog() as cl:
+        piff.piffify(config, cl.logger)
+    assert "WARNING: Items ['reserve_frac'] should now be in the 'select' field" in cl.output
+    psf = piff.read(psf_file)
+    assert type(psf.model) is piff.Gaussian
+    assert type(psf.interp) is piff.Mean
+    nreserve = len([s for s in psf.stars if s.is_reserve])
+    ntot = len(psf.stars)
+    assert nreserve == 2
+    assert ntot == 10
+
+
 @timer
 def test_model():
     """Test Model base class
@@ -577,6 +603,7 @@ def test_psf():
     np.testing.assert_raises(NotImplementedError, psf.drawStar, star)
     np.testing.assert_raises(NotImplementedError, psf.drawStarList, [star])
     np.testing.assert_raises(NotImplementedError, psf._drawStar, star)
+    np.testing.assert_raises(NotImplementedError, psf._getProfile, star)
 
     # Invalid to read a type that isn't a piff.PSF type.
     # Mock this by pretending that SingleChip is the only subclass of PSF.
@@ -586,56 +613,6 @@ def test_psf():
     with mock.patch('piff.util.get_all_subclasses', return_value=[piff.SingleChipPSF]):
         np.testing.assert_raises(ValueError, piff.PSF.read, filename)
 
-@timer
-def test_extra_interp():
-    # Test that specifying extra_interp_properties works properly
-    # TODO: This is a very bare bones test of the interface.  There is basically no test of
-    #       this functionality at all yet.  TBD!
-
-    sigma = 1.3
-    g1 = 0.23
-    g2 = -0.17
-    psf = galsim.Gaussian(sigma=sigma).shear(g1=g1, g2=g2)
-    wcs = galsim.JacobianWCS(0.26, 0.05, -0.08, -0.29)
-    image = galsim.Image(64,64, wcs=wcs)
-    psf.drawImage(image, method='no_pixel')
-
-    # use g-i color as an extra property for interpolation.
-    props = dict(gi_color=0.3)
-    print('props = ',props)
-    star = piff.Star(piff.StarData(image, image.true_center, properties=props), None)
-
-    model = piff.Gaussian(fastfit=True, include_pixel=False)
-    interp = piff.Mean()
-    psf = piff.SimplePSF(model, interp, extra_interp_properties=['gi_color'])
-    assert psf.extra_interp_properties == ['gi_color']
-
-    # Note: Mean doesn't actually do anything useful with the extra properties, so this
-    #       isn't really testing anything other than that the code doesn't completely break.
-    pointing = galsim.CelestialCoord(-5 * galsim.arcmin, -25 * galsim.degrees)
-    psf.fit([star], wcs={0 : wcs}, pointing=pointing)
-
-    # Not much of a check here.  Just check that it actually draws something with flux ~= 1
-    im = psf.draw(x=5, y=7, gi_color=0.3)
-    np.testing.assert_allclose(im.array.sum(), 1.0, rtol=1.e-3)
-
-    # Check missing or extra properties
-    with np.testing.assert_raises(TypeError):
-        psf.draw(x=5, y=7)
-    with np.testing.assert_raises(TypeError):
-        psf.draw(x=5, y=7, gi_color=0.3, ri_color=3)
-
-    # No stars raises an error.  (This can happen in practice if all stars are excluded on input.)
-    with np.testing.assert_raises(RuntimeError):
-        psf.fit([], wcs={0 : wcs}, pointing=pointing)
-
-    # Also for SingleChipPSf
-    psf2 = piff.SingleChipPSF(psf, extra_interp_properties=['gi_color'])
-    assert psf2.extra_interp_properties == ['gi_color']
-
-    with np.testing.assert_raises(TypeError):
-        psf2.draw(x=5, y=7, chipnum=0)
-
 @timer
 def test_load_images():
     """Test the load_images function
@@ -657,6 +634,8 @@ def test_load_images():
         bounds = galsim.BoundsI(int(x-31), int(x+32), int(y-31), int(y+32))
         psf.drawImage(image[bounds], center=galsim.PositionD(x,y), method='no_pixel')
     image.addNoise(galsim.GaussianNoise(rng=galsim.BaseDeviate(1234), sigma=1e-6))
+    sky = 10.
+    image += sky
     image_file = os.path.join('output','test_load_images_im.fits')
     image.write(image_file)
 
@@ -669,7 +648,9 @@ def test_load_images():
 
     # Make star data
     config = { 'image_file_name' : image_file,
-               'cat_file_name': cat_file }
+               'cat_file_name': cat_file,
+               'sky': 10
+             }
     orig_stars, wcs, pointing = piff.Input.process(config, logger)
 
     # Fit these with a simple Mean, Gaussian
@@ -686,14 +667,24 @@ def test_load_images():
     for star in psf2.stars:
         np.testing.assert_array_equal(star.image.array, 0.)
 
-    loaded_stars = piff.Star.load_images(psf2.stars, image_file)
+    # First the old API:
+    with CaptureLog() as cl:
+        loaded_stars = piff.Star.load_images(psf2.stars, image_file, logger=cl.logger)
+    assert "WARNING: The Star.load_images function is deprecated." in cl.output
     for star, orig in zip(loaded_stars, psf.stars):
         np.testing.assert_array_equal(star.image.array, orig.image.array)
 
-    # Can optionally supply sky to subtract
-    loaded_stars = piff.Star.load_images(psf2.stars, image_file, sky=10)
+    # Can optionally supply a different sky to subtract
+    # (This is not allowed by the new API, but it probably doesn't make much sense.)
+    loaded_stars = piff.Star.load_images(psf2.stars, image_file, sky=2*sky)
     for star, orig in zip(loaded_stars, psf.stars):
-        np.testing.assert_array_equal(star.image.array, orig.image.array-10)
+        np.testing.assert_array_equal(star.image.array, orig.image.array - sky)
+
+    # Now the new API:
+    psf2 = piff.read(psf_file, logger)  # Reset stars to not have images in them.
+    loaded_stars = piff.InputFiles(config).load_images(psf2.stars)
+    for star, orig in zip(loaded_stars, psf.stars):
+        np.testing.assert_array_equal(star.image.array, orig.image.array)
 
 @timer
 def test_draw():
@@ -851,6 +842,82 @@ def test_draw():
         assert im10.bounds == im1.bounds
         np.testing.assert_allclose(im10.array, im4.array, rtol=1.e-14, atol=1.e-14)
 
+def test_depr_select():
+    # A bunch of keys used to be allowed in input, but now belong in select.
+    # Check that the old way still works, but gives a warning.
+
+    # This is the input from test_stars in test_input.py
+    dir = 'input'
+    image_file = 'test_input_image_00.fits'
+    cat_file = 'test_input_cat_00.fits'
+    psf_file = os.path.join('output','test_depr_select.fits')
+
+    config = {
+        'input' : {
+                'dir' : dir,
+                'image_file_name' : image_file,
+                'cat_file_name' : cat_file,
+                'weight_hdu' : 1,
+                'sky_col' : 'sky',
+                'gain_col' : 'gain',
+                'use_partial' : True,
+                'nstars': 15,  # Just to make the test faster
+             },
+        'select': {
+                'max_snr' : 200,
+                'min_snr' : 20,
+                'hsm_size_reject' : 20,
+                'max_edge_frac': 0.25,
+                'stamp_center_size': 10,
+                'max_mask_pixels' : 20,
+                'reserve_frac' : 0.1,
+                'seed': 1234,
+        },
+        'psf': {
+            'model' : {'type': 'Gaussian'},
+            'interp' : {'type': 'Mean'},
+        },
+        'output' : { 'file_name' : psf_file },
+    }
+
+    logger = piff.config.setup_logger(log_file='output/test_depr_select.log')
+
+    # This is the new API
+    print('config = ',config)
+    piff.piffify(config, logger)
+    psf1 = piff.read(psf_file)
+    im1 = psf1.draw(x=23, y=34, stamp_size=16)
+    print('len1 = ',len(psf1.stars))
+
+    # This is the old API
+    config['input'].update(config['select'])
+    del config['select']
+    config = galsim.config.CleanConfig(config)
+    print('config = ',config)
+    with CaptureLog(level=1) as cl:
+        piff.piffify(config, cl.logger)
+    assert "WARNING: Items [" in cl.output
+    assert "] should now be in the 'select' field of the config file." in cl.output
+    psf2 = piff.read(psf_file)
+    print('len2 = ',len(psf2.stars))
+    assert len(psf1.stars) == len(psf2.stars)
+    im2 = psf2.draw(x=23, y=34, stamp_size=16)
+    np.testing.assert_allclose(im2.array, im1.array)
+
+    # Also ok for some items to be in select, but erroneously put some in input.
+    config['input']['min_snr'] = config['select'].pop('min_snr')
+    config['input']['max_snr'] = config['select'].pop('max_snr')
+    config = galsim.config.CleanConfig(config)
+    print('config = ',config)
+    with CaptureLog(level=1) as cl:
+        piff.piffify(config, cl.logger)
+    assert "WARNING: Items ['max_snr', 'min_snr'] should now be in the 'select' field of the config file." in cl.output
+    psf3 = piff.read(psf_file)
+    print('len3 = ',len(psf3.stars))
+    assert len(psf1.stars) == len(psf3.stars)
+    im3 = psf3.draw(x=23, y=34, stamp_size=16)
+    np.testing.assert_allclose(im3.array, im1.array)
+
 
 if __name__ == '__main__':
     test_Gaussian()
