@@ -64,13 +64,14 @@ def test_twodstats():
     jcenter = 2000
     # the average value in the bin should match up with the model for the average coordinates
     sigma, g1, g2 = psf_model(icen, jcen, icenter, jcenter)
-    sigma_average = stats.twodhists['T'][v_i, u_i]
+    T = 2*sigma**2
+    T_average = stats.twodhists['T'][v_i, u_i]
     g1_average = stats.twodhists['g1'][v_i, u_i]
     g2_average = stats.twodhists['g2'][v_i, u_i]
     # assert equal to 4th decimal
-    print('sigma, g1, g2 = ',[sigma,g1,g2])
-    print('av sigma, g1, g2 = ',[sigma_average,g1_average,g2_average])
-    np.testing.assert_almost_equal([sigma, g1, g2], [sigma_average, g1_average, g2_average],
+    print('T, g1, g2 = ',[T,g1,g2])
+    print('av T, g1, g2 = ',[T_average,g1_average,g2_average])
+    np.testing.assert_almost_equal([T, g1, g2], [T_average, g1_average, g2_average],
                                    decimal=2)
 
     # Test the plotting and writing
@@ -475,10 +476,11 @@ def test_shapestats_config():
 
     # test their characteristics
     sigma = 1.3  # (copied from setup())
+    T = 2*sigma**2
     g1 = 0.23
     g2 = -0.17
-    np.testing.assert_array_almost_equal(sigma, shapeStats.T, decimal=4)
-    np.testing.assert_array_almost_equal(sigma, shapeStats.T_model, decimal=3)
+    np.testing.assert_array_almost_equal(T, shapeStats.T, decimal=4)
+    np.testing.assert_array_almost_equal(T, shapeStats.T_model, decimal=3)
     np.testing.assert_array_almost_equal(g1, shapeStats.g1, decimal=4)
     np.testing.assert_array_almost_equal(g1, shapeStats.g1_model, decimal=3)
     np.testing.assert_array_almost_equal(g2, shapeStats.g2, decimal=4)
@@ -556,6 +558,7 @@ def test_starstats_config():
     # if use all stars, no randomness
     np.testing.assert_array_equal(starStats.stars[3].image.array, orig_stars[3].image.array)
     np.testing.assert_array_equal(starStats.indices, np.arange(len(orig_stars)))
+    starStats.plot()  # Make sure this runs without error and in finite time.
 
     # check nplot = 0
     starStats = piff.StarStats(nplot=0)
@@ -564,6 +567,7 @@ def test_starstats_config():
     # if use all stars, no randomness
     np.testing.assert_array_equal(starStats.stars[3].image.array, orig_stars[3].image.array)
     np.testing.assert_array_equal(starStats.indices, np.arange(len(orig_stars)))
+    starStats.plot()  # Make sure this runs without error.
 
     # rerun with adjust stars and see if it did the right thing
     # first with starstats == False
@@ -642,7 +646,7 @@ def test_hsmcatalog():
     for col in ['ra', 'dec', 'x', 'y', 'u', 'v',
                 'T_data', 'g1_data', 'g2_data',
                 'T_model', 'g1_model', 'g2_model',
-                'flux', 'reserve']:
+                'flux', 'reserve', 'flag_truth', 'flag_model']:
         assert len(data[col]) == 10
     true_data = fitsio.read(cat_file)
 
@@ -657,6 +661,10 @@ def test_hsmcatalog():
     np.testing.assert_allclose(data['g1_model'], data['g1_data'], rtol=1.e-4)
     np.testing.assert_allclose(data['g2_model'], data['g2_data'], rtol=1.e-4)
 
+    # On this file, no hsm errors
+    np.testing.assert_array_equal(data['flag_truth'], 0)
+    np.testing.assert_array_equal(data['flag_model'], 0)
+
     image = galsim.fits.read(image_file)
     world = [image.wcs.toWorld(galsim.PositionD(x,y)) for x,y in zip(data['x'],data['y'])]
     np.testing.assert_allclose(data['ra'], [w.ra.deg for w in world], rtol=1.e-4)
@@ -704,6 +712,7 @@ def test_bad_hsm():
     shape_file = os.path.join('output','bad_hsm_shape.pdf')
     star_file = os.path.join('output','bad_hsm_star.pdf')
     hsm_file = os.path.join('output','bad_hsm_hsm.fits')
+    sizemag_file = os.path.join('output','bad_hsm_sizemag.png')
 
     stamp_size = 25
 
@@ -750,6 +759,10 @@ def test_bad_hsm():
                     'type': 'Star',
                     'file_name': star_file,
                 },
+                {
+                    'type': 'SizeMag',
+                    'file_name': sizemag_file,
+                },
                 {
                     'type': 'HSMCatalog',
                     'file_name': hsm_file,
@@ -789,9 +802,23 @@ def test_bad_hsm():
     assert len(psf.stars) == 1
     assert psf.nremoved == 7    # There were 8 to start.
 
-    for f in [twodhist_file, rho_file, shape_file, star_file, hsm_file]:
+    for f in [twodhist_file, rho_file, shape_file, star_file, sizemag_file, hsm_file]:
         assert os.path.exists(f)
 
+    # Check hsm file with bad measurements
+    # The one star that was left still fails hsm measurement here.
+    data = fitsio.read(hsm_file)
+    for col in ['ra', 'dec', 'x', 'y', 'u', 'v',
+                'T_data', 'g1_data', 'g2_data',
+                'T_model', 'g1_model', 'g2_model',
+                'flux', 'reserve', 'flag_truth', 'flag_model']:
+        assert len(data[col]) == 1
+    print('flag_truth = ',data['flag_truth'])
+    print('flag_model = ',data['flag_model'])
+    np.testing.assert_array_equal(data['flag_truth'], 7)
+    np.testing.assert_array_equal(data['flag_model'], 7)
+
+
 @timer
 def test_base_stats():
     """Test the base Stats class.
